<# PowerCLI Cookbook - Chapter 2 Code #> 
# Script to mass configure ESXi hosts
# Step 1 - Store credentials for ESXi hosts
# Step 2 - Set a list of target ESXi hosts and IP settings
# Step 3 - Write a ForEach loop to iterate through hosts
# Step 4 - Connect to ESXi host
# Step 5 - Obtain a VMHost object to use for configuration
# Step 6 - Join the ESXi system to Active Directory
# Step 7 - Enable services on the ESXi host & set firewall
# Step 8 - Configure the network settings
# Step 9 - Configure NFS & iSCSI settings
# Step 10 - Join hosts to vCenter
# Step 11 - Rescan for storage changes
# Step 12 - Configure persistent syslog storage

# Step 1 - Store credentials for ESXi hosts
$esxiCreds = Get-Credential

# Step 2 - Set a list of target ESXi hosts
$esxiTargets = "192.168.0.241","192.168.0.242", "192.168.0.243", "192.168.0.244"
	$vMotionNetwork = "192.168.50."
	$storageNetwork = "192.168.100."
	$ftlogNetwork = "192.168.200."

# Step 3 - Write a ForEach loop to iterate through the hosts
ForEach ($hostname in $esxiTargets) {

	# Step 4 - Connect to ESXi host
	$connectedHost = connect-viserver $hostname -Credential $esxiCreds

	# Step 5 - Obtain a VMHost object to use for configuration
	$esxihost = Get-VMHost $hostname

	# Step 6 - Join the ESXi system to Active Directory
	$esxihost | Get-VMHostAuthentication | Set-VMHostAuthentication -JoinDomain -Domain domain.local -user username -password ***** -Confirm:$false

	# Step 7 - Enable services on the ESXi host & set firewall
	$esxihost | Get-VMHostService | where { $_.key -eq "TSM-SSH" } | Set-VMHostService -Policy "On" -Confirm:$false
	$esxihost | Get-VMHostService | where { $_.key -eq "TSM-SSH" } | Start-VMHostService -Confirm:$false

	# Step 8 - Configure the network settings
    $esxihost | Get-VirtualPortGroup -Name "VM Network" | Remove-VirtualPortGroup -Confirm:$false 

    # Assemble IP addresses needed for server
    $esxihost_ipaddress = $esxihost | Get-VMHostNetworkAdapter -name vmk0
    $lastOctet = $esxihost_ipaddress.IP.Split(".")[3]
    $vmotionIP = $vMotionNetwork + $lastOctet
    $storageIP = $storageNetwork + $lastOctet
    $ftlogIP = $ftlogNetwork + $lastOctet

    # Create new vmkernel port for vMotion Network
    $esxihost | New-VMHostNetworkAdapter -PortGroup "VMotion Network" -VirtualSwitch vSwitch0 -IP $vmotionIP -SubnetMask 255.255.255.0 -VMotionEnabled $true
    $esxihost | Get-VirtualPortGroup -Name "VMotion Network" | Set-VirtualPortGroup -VlanID 50

    # Create new virtual switch for Storage and FT Logging
    $esxihost | New-VirtualSwitch -Name vSwitch1 -Nic vmnic2

    # Create vmkernel ports for Storage and FT Logging
    $esxihost | New-VMHostNetworkAdapter -PortGroup "Storage Network" -VirtualSwitch vSwitch1 -IP $storageIP -SubnetMask 255.255.255.0 -VsanTrafficEnabled $true
    $esxihost | Get-VirtualPortGroup -Name "Storage Network" | Set-VirtualPortGroup -VlanID 100
    $esxihost | New-VMHostNetworkAdapter -PortGroup "FT Logging Network" -VirtualSwitch vSwitch1 -IP $ftlogIP -SubnetMask 255.255.255.0 -FaultToleranceLoggingEnabled $true
    $esxihost | Get-VirtualPortGroup -Name "FT Logging Network" | Set-VirtualPortGroup -VlanID 200

    # Create new Virtual Switch for Virtual Machines
    $esxihost | New-VirtualSwitch -Name vSwitch2 -Nic vmnic3

    # Create Port Groups for Virtual Machines
    New-VirtualPortGroup -Name "Infrastructure Network" -VirtualSwitch vSwitch2 -VLanId 1 -Server $connectedHost
    New-VirtualPortGroup -Name "Application Network" -VirtualSwitch vSwitch2 -VLanId 2 -Server $connectedHost

	# Step 9 - Configure NFS & iSCSI settings
	# Connect NFS datastore
	$esxihost | New-Datastore -Nfs -Name DataStoreName -Path /data1/export -NfsHost nfsserver.domain.local

	# Configure iSCSI Settings
	$esxihost | Get-VMHostStorage | Set-VMHostStorage -SoftwareIScsiEnabled $true
    $iSCSIhba = $esxihost | Get-VMHostHba -Type iScsi
	New-IScsiHbaTarget -IScsiHba $iSCSIhba -Address $target -ChapType Required -ChapName vSphere -ChapPassword Password1
	$esxcli = Get-ESXCLI -VMHost $esxihost
	$esxcli.iscsi.networkportal.add($iscsihba, $true,"vmk2")
	Disconnect-VIServer -Server $connectedHost -Confirm:$false
} 