<# PowerCLI Cookbook - Chapter 6 Code #>

<# Setting reservations and limits for resource pools #>
Get-VMResourceConfiguration TTYLinux1
Get-VMResourceConfiguration TTYLinux1 | Set-VMResourceConfiguration -MemReservationMB 25 -CpuLimitMhz 1024
Set-ResourcePool -ResourcePool (Get-ResourcePool -Name Production) -MemReservationGb 1
Set-ResourcePool -ResourcePool (Get-ResourcePool -Location BigCluster -Name Production) -MemReservationGB 1
Set-ResourcePool -ResourcePool (Get-ResourcePool -Location BigCluster -Name Production) -MemReservationMB 1024
Set-ResourcePool -ResourcePool (Get-ResourcePool -Location BigCluster -Name Production) -CpuReservationMhz 512
Set-ResourcePool -ResourcePool (Get-ResourcePool -Location BigCluster -Name Production) -CpuLimitMhz 2048 -MemLimitGB 4

<# Creating a custom attribute with a number of shares per VM on each resource pool #>
Get-CustomAttribute| Select *
New-CustomAttribute -Name SharesPerVM -TargetType ResourcePool
Set-Annotation -Entity (Get-ResourcePool Production) -CustomAttribute SharesPerVM -Value 100
Set-Annotation -Entity (Get-ResourcePool Development) -CustomAttribute SharesPerVM -Value 50
Get-ResourcePool | Select *

<# Functions for Set & Get ResourcePoolShareValues #>
function Set-ResourcePoolShareValues
{

  <#
  .SYNOPSIS
  Sets the custom share value of each resource pool based 
  on the number of virtual machines in the pool
  .DESCRIPTION
  Calculates the number of shares per resource pool as the 
  number of virtual machines multiplied by the SharesPerVM 
  custom attribute stored on each virtual machine.  Prompts 
  user to input a SharesPerVM value if missing or emails 
  administrators if run as a scheduled task.  
  .EXAMPLE
  Set-ResourcePoolShareValues
  #>

  param (
    [Parameter(Mandatory=$False,
    ValueFromPipeline=$False,
    ValueFromPipelineByPropertyName=$False,
    HelpMessage='Name:')]
    [Boolean]$RunAsTask = $False
  )

  process {
    $Pools = Get-ResourcePool | Where {$_.Name -ne "Resources"}
    ForEach($Pool in $Pools) {
        $SharesPerVM = $null
        $VmCount = Get-VM -Location $Pool | Where {$_.PowerState -eq "PoweredOn"} |  Measure
        If ($Pool.CustomFields.Item("SharesPerVM") -eq "" -and $RunAsTask -eq $False) {
          [int]$SharesPerVM = Read-Host "Missing SharePerVM value for $($Pool.Name) resource pool.  How many shares per VM in the $($Pool.Name) resource pool?"
          Set-Annotation -Entity $Pool -CustomAttribute SharesPerVM -Value $SharesPerVM
        } Else {
          [int]$SharesPerVM = $Pool.CustomFields.Item("SharesPerVM")
        } <# End If #>
        If ($SharesPerVM -ne $null) {
          $PoolShares = $VmCount.Count * $SharesPerVM
          Set-ResourcePool -ResourcePool $Pool -CpuSharesLevel:Custom -NumCpuShares $PoolShares -MemSharesLevel:Custom -NumMemShares $PoolShares -Confirm:$false
        } Else {
            $msg = New-Object Net.Mail.MailMessage
            $smtpServer = New-Object Net.Mail.SmtpClient("hostname")
            $msg.From = "fromaddress@yourcompany.com"
            $msg.To.Add("admin@yourcompany.com")
            $msg.Subject = "Set-ResourcePoolShareValues - Missing SharePerVM value for resource pool"
            $msg.Body = "The resource pool $($pool.Name) does not have a SharesPerVM allocation saved on the pool.  Please run Set-ResourcePoolShareValues interactively to set and save a value." 
            $smtpServer.Send($msg)
        } <# End If #>
     } <# End ForEach #>
  } <# End process #>
} <# End Function #>

function Get-ResourcePoolShareValues { 

  <#
  .SYNOPSIS
  Returns list of resource pools with their share values
  .DESCRIPTION
  This function retrieves the set values for CPU and Memory shares along with the SharesPerVM custom attribute, the number of virtual machines running in the pool and the computed value of shares to verify the settings
  .EXAMPLE
  Get-ResourcePoolShareValues
  #>

  param (
  )

  process {
    Get-ResourcePool | Where {$_.Name -ne "Resources"} | Select Name, CPUSharesLevel, NumCPUShares, MemSharesLevel, NumMemShares, @{N="Owner";E={(Get-View $_.ExtensionData.Owner).Name}}, @{N="SharesPerVM";E={$_.CustomFields.Item('SharesPerVM')}}, @{N="ActiveVMs";E={(Get-VM -Location $_ | Where {$_.PowerState -eq 'PoweredOn'} | Measure).Count}},  @{N="ComputedShares";E={[int]$_.CustomFields.Item('SharesPerVM') * [int](Get-VM -Location $_ | Where {$_.PowerState -eq 'PoweredOn'} | Measure).Count} } | FT
  } <# End process #>

}
