<# PowerCLI Cookbook - Chapter 7 Code #>
<# Getting alerts from vSphere environment #>

Get-VM | Where {$_.ExtensionData.TriggeredAlarmState -ne $null} | Select Name, @{N="AlarmName";E={(Get-View $_.ExtensionData.TriggeredAlarmState.Alarm).Info.Name}}, 
@{N="AlarmDescription";E={(Get-View $_.ExtensionData.TriggeredAlarmState.Alarm).Info.Description}}, @{N="TimeTriggered"; E={$_.ExtensionData.TriggeredAlarmState.Time}}, @{N="AlarmOverallStatus"; E={$_.ExtensionData.TriggeredAlarmState. OverallStatus}}

<# Basics of formatting output from PowerShell objects #>

Get-VM | Select * | FT Name, PowerState, @{Label="Num CPUs"; Expression={($_.NumCpu).ToString()}; Alignment="left"}, @{Label="MemoryGB"; Expression={$_.MemoryGB}; FormatString="N2"; Alignment="left"}

Get-Datastore | Select Name, @{N="FreePercent";E={"{0,15:p}" -f ($_.FreeSpaceGB / $_.CapacityGB)}}

<# Sending output to CSV and HTML #>

$VMs = Get-VM | Select Name, NumCPU, MemoryGB, Description, @{N="Hostname";E={$_.ExtensionData.Guest.HostName}}, @{N="IP";E={$_.ExtensionData.Guest.IPAddress}}, @{N="OS";E={$_.ExtensionData.Guest.GuestFullName}}
$HTMLBody = $VMs | ConvertTo-HTML 
$css = "<style> body { font-family: Verdana, sans-serif; font-size: 14px; color: #666; background: #FFF; } table{ width:100%; border-collapse:collapse; } table td, table th { border:1px solid #333; padding: 4px; } table th { text-align:left; padding: 4px; background-color:#BBB; color:#FFF;} </style>"
$HTMLBody = $VMs | ConvertTo-HTML -PreContent $css
$HTMLBody = $VMs | ConvertTo-HTML -PreContent $css -PostContent "<div><strong>Generated:</strong> $(Get-Date)</div>"
$tempdir = [environment]::getfolderpath("mydocuments")
$VMs | Export-CSV $tempdir\VM_Inventory.csv
$msg = New-Object Net.Mail.MailMessage
$attachment = new-object Net.Mail.Attachment("$tempdir\VM_Inventory.csv")
$smtpServer = New-Object Net.Mail.SmtpClient("hostname")
$msg.From = "fromaddress@yourcompany.com"
$msg.To.Add("admin@yourcompany.com")
$msg.Subject = "Weekly VM Report"
$msg.Body = $HTMLBody | Out-String
$msg.Attachments.Add($attachment)
$msg.IsBodyHtml = $true
$smtpServer.Send($msg)
Remove-Item $tempdir\VM_Inventory.csv -Confirm:$false

<# Reporting VM objects created during a time period from VI Events object #>

Get-VIEvent -Start (Get-Date).AddMonths(-1) -MaxSamples 25000 | Where {$_.GetType().Name -match "(VmCreatedEvent|VmClonedEvent)"} | Select @{N="Type";E={$_.GetType().Name}}, @{N="VMName";E={$_.Vm.Name}}, FullFormattedMessage, CreatedTime, UserName

<# Setting custom properties on VMs to add useful context to your virtual machines #>

New-CustomAttribute -TargetType VirtualMachine -Name CreatedBy
New-CustomAttribute -TargetType VirtualMachine -Name CreateDate 

ImportSystemModules
$Events = Get-VIEvent -Start (Get-Date).AddDays(-7) -MaxSamples 50000 | Where {$_.GetType().Name -match "(VmCreatedEvent|VmClonedEvent)"}
ForEach ($Event in $Events) {
  $VM = Get-VM -Name $Event.Vm.Name
  $VM | Set-Annotation -CustomAttribute "CreateDate" -Value $Event.CreatedTime
  $User = (($Event.UserName.split("\"))[1])
  $DisplayName = (Get-AdUser $User).Name
  $VM | Set-Annotation -CustomAttribute "CreatedBy" -Value $DisplayName
} <# End ForEach #>

<# Using PowerShell native capabilities to schedule reports #>

$WeeklySundayAt6AM = New-JobTrigger -Weekly -At "6:00 AM" -DaysOfWeek Sunday –WeeksInterval 1
$Options = New-ScheduledJobOption –WakeToRun –StartIfIdle –MultipleInstancesPolicy Queue

Register-ScheduledJob -Name "Cleanup 30 Day Snapshots" -ScriptBlock { Add-PSSnapIn VMware.VimAutomation.Core; Connect-VIServer servers; Get-Snapshot -VM * | Where {$_.Created -LT (Get-Date).AddDays(-30)} | Remove-Snapshot -Confirm:$false} -Trigger $WeeklySundayAt6AM -ScheduledJobOption $Options

Register-ScheduledJob -Name "Cleanup 30 Day Snapshots" -ScriptBlock { Add-PSSnapIn VMware.VimAutomation.Core; Connect-ViServer server; Import-Module 30DaySnaps; Remove-30DaySnaps -VM *;} -Trigger $WeeklySundayAt6AM - ScheduledJobOption $Options