<# PowerCLI Cookbook - Chapter 9 Code File #> 

<# If you do not have VMs to work with for DRS groups, run the following:
New-VM -Template "WinTemplate" -Name "DC01" -ResourcePool "Production" -DiskStorageFormat Thin -Location "App Servers"
New-VM -Template "WinTemplate" -Name "DC02" -ResourcePool "Production" -DiskStorageFormat Thin -Location "App Servers"
New-VM -Template "WinTemplate" -Name "App01" -ResourcePool "Production" -DiskStorageFormat Thin -Location "App Servers"
New-VM -Template "WinTemplate" -Name "DB01" -ResourcePool "Production" -DiskStorageFormat Thin -Location "App Servers"
#>

<# Get DRS Recommendations #> 
Get-DRSRecommendation

<# Get DRS Recommendations for a specific cluster#> 
Get-DRSRecommendation -Cluster "BigCluster"
Get-Cluster "BigCluster" | Get-DRSRecommendation

<# Get DRS Recommendations for a specific cluster and specific priorities #> 
Get-DRSRecommendation -Cluster "BigCluster" -Priority 1,2,3  

<# Return only a specific recommendation identified by its key value #>
Get-DRSRecommendation | Where {$_.Key -eq "255"}

<# Apply Specific DRS Recommendations in a manual DRS cluster #> 
Get-DRSRecommendation | Where {$_.Name -eq ""} | Apply-DRSRecommendation
    <# For vSphere 6, change to Invoke-DRSRecommendation #>
    Get-DRSRecommendation | Where {$_.Name -eq ""} | Invoke-DRSRecommendation

<# Apply DRS Recommendations with a specific priority in a manual DRS cluster #> 
Get-DRSRecommendation -Cluster "BigCluster" -Priority 1,2,3 | Apply-DRSRecommendation
    <# For vSphere 6, change to Invoke-DRSRecommendation #>
    Get-DRSRecommendation -Cluster "BigCluster" -Priority 1,2,3 | Invoke-DRSRecommendation

<# Using Regular Expression to Return Specific hosts that match criteria and put them in maintenance mode #> 
Get-VMHost esx[135].domain.local | Set-VMHost -State "Maintenance"

<# Maintenance Mode on node in a Manual DRS cluster causes DRS recommendations that need to be applied #> 
Get-DRSRecommendation -Cluster BigCluster | Apply-DRSRecommendation
    <# For vSphere 6, change to Invoke-DRSRecommendation #>
    Get-DRSRecommendation -Cluster BigCluster | Invoke-DRSRecommendation

<# Retrieve any Powered Off VMs after host is in maintenance mode and move them to another host #> 
Get-VMHost esx[135].domain.local | Get-VM | Move-VM -Destination esx2.domain.local -Confirm:$false

<# Create DRS Rule for Separation of VMs #> 
New-DRSRule -Name "Separate Active Directory" -Cluster BigCluster -KeepTogether:$false -VM DC01,DC02

<# Create DRS Rule for Keeping VMs Together #>
New-DRSRule -Name "App One" -Cluster BigCluster -KeepTogether:$true -VM App01,DB01

<# Retrieve List of DRS Rules for a Specific Cluster #>
Get-DRSRule -Cluster BigCluster
Get-DRSRule -Cluster Bigcluster | Select Name, Enabled, KeepTogether, @{N="VM";E={ ForEach ($VM in $_.VMIds) { (Get-VM -Id $vm).Name } } }

<# Obtain the MoREF of a VM #>
(Get-VM -Name TTYLinux1).ExtensionData.MoREF

<# Use MoREF in an Object's Extension Data to Lookup Name of the Referenced Datastore #> 
Get-Datastore -ID ((Get-VM -Name TTYLinux1).ExtensionData.Datastore)

<# Get a Cluster View Object #>
Get-Cluster "BigCluster" | Get-View

<# Get List of DRS Groups on a Cluster #> 
(Get-Cluster "BigCluster" | Get-View).ConfigurationEx.Group

<# Define a New VM DRS Group #> 
$group = New-Object VMware.Vim.ClusterGroupSpec
$group.Info = New-Object VMware.Vim.ClusterVmGroup 
$group.Info.Name = "TTYLinux VMs"
$vms = Get-VM "TTYLinux*"
ForEach ($vm in $vms) {
  $group.Info.Vm += $vm.ExtensionData.MoRef
}
$cluster = Get-Cluster "BigCluster" | Get-View
$spec = New-Object VMware.Vim.ClusterConfigSpecEx
$spec.GroupSpec = $group
$cluster.ReconfigureComputeResource_Task($spec,$true)

<# List Group Members of a DRS Group #>
(Get-Cluster "BigCluster" | Get-View).ConfigurationEx.Group | Select Name, @{N='VMs';E={ForEach ($vm in $_.Vm) { (Get-VM -Id $vm).Name }}}
(Get-Cluster "BigCluster" | Get-View).ConfigurationEx.Group | Select Name, @{N='VMs';E={ForEach ($vm in $_.Vm) { (Get-VM -Id $vm).Name }}}, @{N='Hosts';E={ForEach ($vmhost in $_.Host) { (Get-VMHost -Id $vmhost).Name }}}

<# Update Members of a VM DRS Cluster #> 
$cluster = Get-Cluster "BigCluster"
$spec = New-Object VMware.Vim.ClusterConfigSpecEx
$group = New-Object VMware.Vim.ClusterGroupSpec
$group.Info = New-Object VMware.Vim.ClusterVmGroup 
$group.Operation = "edit"
$group.Info.Name = "TTYLinux VMs"
$vms = Get-VM TTYLinux1
ForEach ($vm in $vms) {
  $group.Info.VM += $vm.ExtensionData.MoREF
}
$spec.GroupSpec += $group
$cluster.ExtensionData.ReconfigureComputeResource_Task( $spec, $true)

<# Setup the PowerShell ISE to do PowerCLI #> 
Add-PSSnapin VMware.VimAutomation.Core
Connect-VIServer vcenter.domain.local

<# Function for Updating VM DRS Groups #> 
function Update-VmDrsGroup {
  <#
  .SYNOPSIS
    Updates the VMs in an existing VM DRS group
  .DESCRIPTION
    Reconfigures an existing virtual machine DRS group to contain the group of virtual machines defined by the  -VM property of the function.
  .PARAMETER Cluster
    The name of the cluster with the DRS group to update
  .PARAMETER Name
    The name of the VM DRS group to update
  .PARAMETER VM
    The list of virtual machines to place in the DRS group.
  .EXAMPLE
    Update-VmDrsGroup -Cluster BigCluster -Name "VM Group" -VM (Get-VM WinVM*)
  .EXAMPLE
    Update-VmDrsGroup -Cluster BigCluster -Name "VM Group" -VM WinVM1,DB1,App1
  #>
  param (
    [parameter(valuefrompipeline = $false, mandatory = $true,
      HelpMessage = "Enter a list of VM names")]
        [PSObject] $VM,
    [parameter(mandatory = $true,
      HelpMessage = "Enter the name of a cluster")]
        [PSObject] $Cluster,
    [parameter(mandatory = $true,
      HelpMessage = "Enter the name of a VM DRS Group")]
        [String] $Name
  )
  process {
    $spec = New-Object VMware.Vim.ClusterConfigSpecEx
    $group = New-Object VMware.Vim.ClusterGroupSpec  
    $group.Info = New-Object VMware.Vim.ClusterVmGroup
    $group.Operation = "edit"
    $group.Info.Name = $Name
    Switch ($Cluster.gettype().name ) {
      "String" { $Cluster = Get-Cluster -Name $Cluster }
      "ClusterImpl" { } 
      "ClusterComputeResource" { $Cluster = Get-Cluster -Id $Cluster.MoRef }
      "ManagedObjectReference" { $Cluster = Get-Cluster -Id $Cluster } 
      default { throw "The data specified for the -Cluster variable does not match the expected types." } 
    }
    if (($VM.gettype().name) -eq "String") { $VM = Get-VM -Name $VM }
    ForEach ($item in $VM) {
        Switch ($item.gettype().name ) {
          "String" { Get-VM -Name $VM | %{ $group.Info.VM += $_.ExtensionData.MoREF }}
          "VirtualMachineImpl" { $group.Info.VM += $item.ExtensionData.MoREF } 
          "VirtualMachine" { $group.Info.VM += $item.MoRef } 
        } 
    }
    if ($group.Info.VM) {
        $spec.GroupSpec += $group
        $cluster.ExtensionData.ReconfigureComputeResource_Task( $spec, $true)
    } else {
        throw "There are no Virtual Machines defined.  Update failed."
    } 
  } 
}

<# Function for Updating Host DRS Groups #> 
function Update-HostDrsGroup {
  <#
  .SYNOPSIS
    Updates the VMs in an existing VM DRS group
  .DESCRIPTION
    Reconfigures an existing virtual machine DRS group to contain the group of virtual machines defined by the -VM property of the function.
  .PARAMETER Cluster
    The name of the cluster with the DRS group to update
  .PARAMETER Name
    The name of the VM DRS group to update
  .PARAMETER VMHost
    The list of hosts to place in the DRS group.
  .EXAMPLE
    PS> Update-HostDrsGroup -Cluster BigCluster -Name "Site Hosts" -VMHost (Get-VMHost ESXi*)
  .EXAMPLE
    PS> Get-VMHost ESXi* | Update-HostDrsGroup -Cluster BigCluster -Name "Site Hosts" 
  #>
  param (
    [parameter(valuefrompipeline = $true, mandatory = $true,
      HelpMessage = "Enter a list of VMHost names")]
        [PSObject] $Host,
    [parameter(mandatory = $true,
      HelpMessage = "Enter the name of a cluster")]
        [PSObject] $Cluster,
    [parameter(mandatory = $true,
      HelpMessage = "Enter the name of a Host DRS Group")]
        [String] $Name
  )
  process {
    $spec = New-Object VMware.Vim.ClusterConfigSpecEx
    $group = New-Object VMware.Vim.ClusterGroupSpec  
    $group.Info = New-Object VMware.Vim.ClusterHostGroup
    $group.Operation = "edit"
    $group.Info.Name = $Name
    Switch ($Cluster.gettype().name ) {
      "String" { $Cluster = Get-Cluster -Name $Cluster }
      "ClusterImpl" { } 
      "ClusterComputeResource" { $Cluster = Get-Cluster -Id $Cluster.MoRef }
      "ManagedObjectReference" { $Cluster = Get-Cluster -Id $Cluster } 
      default { throw "The data specified for the -Cluster variable does not match the expected types." } 
    }
    Switch ($VMHost.gettype().name ) {
      "String" { Get-VMHost -Name $VMHost | %{ $group.Info.Host += $_.ExtensionData.MoREF }}
      "VmHostImpl" { $group.Info.Host += $VMHost.ExtensionData.MoREF } 
      "HostSystem" { $group.Info.Host += $VMHost.MoRef } 
    } 
    if ($group.Info.Host) {
    $spec.GroupSpec += $group
      $cluster.ExtensionData.ReconfigureComputeResource_Task($spec, $true)
    } else {
       throw "There are no hosts defined.  Update failed."
    } 
  } 
}

<# You will need to create five groups in the vCenter Client to use these rules and updates.  Place the vCenter VM in each of the groups, since you cannot create an empty group.  
If the groups contain different members after writing the rules, the update worked successfully. The groups to be created are Database Servers, iSCSICluster VMs, Datastore1 VMs, 
WinVMs, and VM Group, on the BigCluster vSphere cluster. #>

<# Sample Rules using Update-VmDrsGroup #>
Update-VmDrsGroup -Name "Production Servers" -Cluster "BigCluster" -VM (Get-ResourcePool "Production" | Get-VM )

Update-VmDrsGroup -Name "iSCSICluster VMs" -Cluster "BigCluster" -VM (Get-DatastoreCluster "iSCSICluster" | Get-VM)

Update-VmDrsGroup -Name "Datastore1 VMs" -Cluster "BigCluster" -VM (Get-VM -Datastore "Datastore1")

Update-VmDrsGroup -Name "WinVMs" -Cluster "BigCluster" -VM WinVM*

Update-VmDrsGroup -Name "VM Group" -Cluster "BigCluster" -VM WinVM1,TTYLinux1,DC02

(Get-Cluster "BigCluster").ExtensionData.ConfigurationEx.Group

<# Audit for VM DRS Groups #>
$group = (Get-Cluster "BigCluster").ExtensionData.ConfigurationEx.Group | Where {$_.Name -eq "VM Group"}
$vm = Get-VM WinVM1,TTYLinux1,DC02
Compare-Object ($group.Vm) ($vm.ExtensionData.MoRef)
Compare-Object ($group.Vm) ($vm.ExtensionData.MoRef) | Select @{N="VM";E={(Get-VM -Id $_.InputObject).Name}}, SideIndicator

$vm = Get-VM TTYLinux*
Compare-Object ($group.Vm) ($vm.ExtensionData.MoRef) | Select @{N="VM";E={(Get-VM -Id $_.InputObject).Name}}, SideIndicator

<# Update Membership if differences found #> 
If (Compare-Object ($group.Vm) ($vm.ExtensionData.MoRef) | Select @{N="VM";E={(Get-VM -Id $_.InputObject).Name}}, SideIndicator) {
  Update-VmDrsGroup -Name "TTYLinux VMs" -Cluster "BigCluster" -VM $vm
} else {
  write-host "There were no differences."
}
